@@ -2,7 +2,7 @@
  * command.c
  *
  * All rights reserved. Copyright (C) 1996-1999 by NARITA Tomio.
- * $Id: command.c,v 1.13 1999/11/02 17:46:22 nrt Exp $
+ * $Id: command.c,v 1.14 2000/06/26 11:36:43 nrt Exp $
  */
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -144,7 +144,7 @@ private void CommandCopyToHistory( str_t *str, int length )
 
 private i_str_t *CommandGetLine( file_t *f, byte prompt )
 {
-  int ch, ptr, iptr, width, index;
+  int ch, ptr, iptr, width, pre_width, index;
   i_str_t *istr;
   byte str[ STRING_SIZE ];
 
@@ -158,16 +158,17 @@ private i_str_t *CommandGetLine( file_t *f, byte prompt )
   iptr = 0;
   istr = NULL;
   width = 0;
+  pre_width = -1;
   for( ; ; ){
     ConsoleFlush();
     ch = ConsoleGetChar();
     switch( ch ){
     case EOF:
-    case BEL:
+    case BEL: /* C-g */
       if( NULL != istr )
 	IstrFree( istr );
       return NULL;
-    case CR:
+    case CR: /* C-m */
       if( NULL == istr )
 	return NULL;
       if( NOSET == istr[ 0 ].charset ){
@@ -183,17 +184,13 @@ private i_str_t *CommandGetLine( file_t *f, byte prompt )
       CommandCopyToHistory( encode_str, ptr );
 
       return istr;
-    case BS:
-/*    case DEL:*/
+    case BS: /* C-h */
       if( 0 == iptr ){
-	if( NULL == istr )
-	  return NULL;
-	if( NOSET == istr[ 0 ].charset ){
+	if( NULL != istr )
 	  IstrFree( istr );
-	  return NULL;
-	}
-	return istr;
+	return NULL;
       }
+backspace:
       iptr--;
       ptr = CODE_SIZE;
       Encode( istr, 0, iptr,
@@ -202,26 +199,40 @@ private i_str_t *CommandGetLine( file_t *f, byte prompt )
       if( ptr > STRING_SIZE )
 	ptr = STRING_SIZE;
       StrCopy( str, encode_str, ptr );
+      pre_width = -1;
       break;
-    case NAK:
+    case NAK: /* C-u */
       if( NULL != istr )
 	IstrFree( istr );
       ptr = 0;
       iptr = 0;
       istr = NULL;
       width = 0;
+      pre_width = -1;
       break;
-    case DLE:
+    case DLE: /* C-p */
       if( --index < 0 )
 	index = HISTORY_SIZE - 1;
       if( FALSE == history[ index ].used )
 	continue;
       Copy( str, history[ index ].str, history[ index ].length );
       ptr = history[ index ].length;
+      pre_width = -1;
       break;
+    case DEL: /* del */
+      if( 0 == ptr ){
+	if( NULL != istr )
+	  IstrFree( istr );
+	return NULL;
+      }
+      /* fall through */
     default:
-      if( ptr >= STRING_SIZE || width >= (f->width - 6) ) /* for CNTRL */
-	continue;
+      if( ptr >= STRING_SIZE || width >= (f->width - 6) ){ /* for CNTRL */
+	if( DEL == ch )
+	  goto backspace;
+	else
+	  continue;
+      }
       str[ ptr++ ] = ch;
     }
 
@@ -243,6 +254,21 @@ private i_str_t *CommandGetLine( file_t *f, byte prompt )
     ConsoleClearRight();
     ConsolePrint( prompt );
     ConsolePrintsStr( encode_str, encode_length );
+
+    if( -1 != pre_width && width < pre_width ){
+      /*
+       * probably character was deleted.
+       * refresh str.
+       */
+      ptr = CODE_SIZE;
+      Encode( istr, 0, iptr,
+	     f->keyboardCodingSystem, TRUE,
+	     encode_str, &ptr );
+      if( ptr > STRING_SIZE )
+	ptr = STRING_SIZE;
+      StrCopy( str, encode_str, ptr );
+    }
+    pre_width = width;
   }
 }
 
@@ -318,7 +344,7 @@ private boolean_t CommandGetNumber( unsigned int *number, int *newCom )
     ch = ConsoleGetChar();
     if( EOF == ch )
       return FALSE;
-    else if( BS == ch /*|| DEL == ch*/ ){
+    else if( BS == ch || DEL == ch ){
       n /= 10U;
       if( 0 == n )
 	return FALSE;
